﻿/**
 * 
 * Find more about the Spinning Wheel function at
 * http://cubiq.org/spinning-wheel-on-webkit-for-iphone-ipod-touch/11
 *
 * Copyright (c) 2009 Matteo Spinelli, http://cubiq.org/
 * Released under MIT license
 * http://cubiq.org/dropbox/mit-license.txt
 * 
 * Version 1.4 - Last updated: 2009.07.09
 * 
 * Horst Klier June 2011 
 * iOS Retina and Android HD Version using devicePixelRatio
 * Added 2 custom buttons
 *  
 */

var SpinningWheel = {
    cellHeight: 44,
	friction: 0.003,
	device: 'i',
	pixelRatio: 1,
	adjustField: '',
	callBack: null,
	onSpin: null,
	slotData: [],


	/**
	 *
	 * Event handler
	 *
	 */

	handleEvent: function (e) {
		if (e.type == 'touchstart') {
//console.log('spinner touchstart')			
			this.lockScreen(e);
			if (e.currentTarget.id == 'sw-cancel' || e.currentTarget.id == 'sw-done' || e.currentTarget.id == 'sw-buttonl' || e.currentTarget.id == 'sw-buttonr') {
				if(this.device=='a'){
					this.tapUp(e);
				}else{
					this.tapDown(e);
				}
			} else if (e.currentTarget.id == 'sw-frame') {
				this.scrollStart(e);
			}
		} else if (e.type == 'touchmove') {
			this.lockScreen(e);
			if (e.currentTarget.id == 'sw-cancel' || e.currentTarget.id == 'sw-done' || e.currentTarget.id == 'sw-buttonl' || e.currentTarget.id == 'sw-buttonr') {
				if(this.device=='i'){
					this.tapCancel(e);
				}
			} else if (e.currentTarget.id == 'sw-frame') {
				this.scrollMove(e);
			}
		} else if (e.type == 'touchend') {
			if (e.currentTarget.id == 'sw-cancel' || e.currentTarget.id == 'sw-done' || e.currentTarget.id == 'sw-buttonl' || e.currentTarget.id == 'sw-buttonr') {
				if(this.device=='i'){
					this.tapUp(e);
				}
			} else if (e.currentTarget.id == 'sw-frame') {
				this.scrollEnd(e);
			}
		} else if (e.type == 'webkitTransitionEnd') {
			if (e.target.id == 'sw-wrapper') {
				this.destroy();
			} else {
				this.backWithinBoundaries(e);
			}
		} else if (e.type == 'orientationchange') {
			//this.onOrientationChange(e); // wird von script.js aufgerufen
		} else if (e.type == 'scroll') {
			this.onScroll(e);
		}
	},


	/**
	 *
	 * Global events
	 *
	 */

	onOrientationChange: function (e) {
		//window.scrollTo(0, 0);
		this.calculatePickerPos();
		this.calculateSlotsWidth();
	},
	
	onScroll: function (e) {
		this.calculatePickerPos();
	},

	lockScreen: function (e) {
		e.preventDefault();
		e.stopPropagation();
	},


	/**
	 *
	 * Initialization
	 *
	 */

	reset: function () {
		this.slotEl = [];

		this.activeSlot = null;
		
		this.swWrapper = undefined;
		this.swSlotWrapper = undefined;
		this.swSlots = undefined;
		this.swFrame = undefined;
	},

	calculateSlotsWidth: function () {
		var div = this.swSlots.getElementsByTagName('div');
		var allWidth,divsWidth,restWidth;
		allWidth=this.swSlots.offsetWidth;
		restWidth=allWidth;
		divsWidth=0;
		for (var i = 0; i < div.length; i += 1) {
			//this.slotEl[i].slotWidth = div[i].offsetWidth;
			divsWidth+=div[i].offsetWidth;
		}
//console.log('allWidth: '+allWidth+'\ndivsWidth: '+divsWidth)			
		for (i = 0; i < div.length; i += 1) {
//console.log(i+': '+div[i].offsetWidth+' > '+(Math.floor(div[i].offsetWidth*(allWidth/divsWidth))))			
			div[i].style.width=(Math.floor(div[i].offsetWidth*(allWidth/divsWidth))-1)+'px'
			this.slotEl[i].slotWidth = div[i].offsetWidth;
			restWidth=restWidth-this.slotEl[i].slotWidth;
//console.log('restWidth='+restWidth)			
			if(i==(div.length-1)&&restWidth>1){
				restWidth--;
				this.slotEl[i].slotWidth=div[i].offsetWidth+restWidth;
				div[i].style.width=this.slotEl[i].slotWidth+'px';
			}
		}
	},
	calculatePickerPos: function () {
		document.getElementById('sw-shield').style.width = window.innerWidth+'px';
		document.getElementById('sw-shield').style.height=window.innerHeight+'px';
		document.getElementById('sw-shield').style.top=window.pageYOffset+'px';
		this.swWrapper.style.top = window.innerHeight + window.pageYOffset + 'px';
		this.swWrapper.style.width = window.innerWidth+'px';
		this.swWrapper.style.left = '0';
		if(window.innerWidth>320*this.pixelRatio){
			this.swWrapper.style.width = (320*this.pixelRatio)+'px';
			this.swWrapper.style.left = ((window.innerWidth-(320*this.pixelRatio))/2)+'px';
			//this.swWrapper.style.WebkitBoxShadow = '1px 1px 10px 5px rgba(0, 0, 0, 1)';
		}		
		if(window.innerHeight>600*this.pixelRatio){
			//this.swWrapper.style.WebkitBoxShadow = '1px 1px 10px 5px rgba(0, 0, 0, 1)';
			this.swWrapper.style.top = (window.innerHeight + window.pageYOffset - (32*this.pixelRatio)) + 'px';		// Place the SW down the actual viewing screen
			if(this.adjustField!=''){
				var newy;
				newy=findPosY(document.getElementById(this.adjustField))
				if((newy+320*this.pixelRatio)<(window.innerHeight + window.pageYOffset - (32*this.pixelRatio))){
					this.swWrapper.style.top = (newy+320*this.pixelRatio) + 'px';
				}
			}
		}
	},

	create: function () {
console.log('spinner create')		
		var i, l, out, ul, div;

		this.reset();	// Initialize object variables
// hk 19.02.2014 DPR kann auch höher sein; Nexus 5 hat 3		
		// hk 30.09.2014 dpr wird nicht mehr benötigt
		/*this.pixelRatio=window.devicePixelRatio;
		if(window.devicePixelRatio>=1.5)this.pixelRatio=1.5;
		if(window.devicePixelRatio>=2)this.pixelRatio=2;
		if(window.devicePixelRatio>=3)this.pixelRatio=3;
		if(window.devicePixelRatio>=4)this.pixelRatio=4;
		this.pixelRatio=window.devicePixelRatio;
		*/
		this.cellHeight=Math.floor(44*this.pixelRatio);

		// Schutzschicht
		div = document.createElement('div');
		div.id = 'sw-shield';
		div.style.top = '0';
		div.style.left = '0';
/*		
		div.style.width = window.innerWidth+'px';
		var n,n2;
		n=document.getElementById('main').offsetHeight;
		n2=document.getElementById('edit').offsetHeight;
		if(n2>n)n=n2;
		div.style.height=n+'px';
		n=document.getElementById('main').offsetWidth;
		n2=document.getElementById('edit').offsetWidth;
		if(n2>n)n=n2;
		div.style.width=n+'px';
	    div.style.opacity=0;
	    //div.style.backgroundColor='#000000';
*/		
		
	    div.style.opacity=0;
	    div.style.position='absolute';
	    //div.style.backgroundColor='#000000';
		div.style.width = window.innerWidth+'px';
		div.style.height=window.innerHeight+'px';
		div.style.top=window.pageYOffset+'px';
		document.body.appendChild(div);

		/////////
		
		// Create the Spinning Wheel main wrapper
		div = document.createElement('div');
		div.id = 'sw-wrapper';
		div.style.top = window.innerHeight + window.pageYOffset + 'px';		// Place the SW down the actual viewing screen
		div.style.width = window.innerWidth+'px';
		div.style.left = '0';

		if(window.innerWidth>320*this.pixelRatio){
			div.style.width = Math.floor(320*this.pixelRatio)+'px';
			div.style.left = Math.floor((window.innerWidth-(320*this.pixelRatio))/2)+'px';
			//div.style.WebkitBoxShadow = '1px 1px 10px 5px rgba(0, 0, 0, 1)';
		}		
		if(window.innerHeight>600*this.pixelRatio){
			//div.style.WebkitBoxShadow = '1px 1px 10px 5px rgba(0, 0, 0, 1)';
			div.style.top = (window.innerHeight + window.pageYOffset - Math.floor(32*this.pixelRatio)) + 'px';		// Place the SW down the actual viewing screen
			if(this.adjustField!=''){
				var newy;
				newy=findPosY(document.getElementById(this.adjustField))
				if((newy+320*this.pixelRatio)<(window.innerHeight + window.pageYOffset - (32*this.pixelRatio))){
					div.style.top = Math.floor(newy+320*this.pixelRatio) + 'px';
				}
			}
		}

		div.style.webkitTransitionProperty = '-webkit-transform, opacity';
		div.style.opacity = 0;
		div.innerHTML = '<div id="sw-header"><div id="sw-cancel">Cancel</' + 'div><div id="sw-buttonl">&lt;Last</' + 'div><div id="sw-buttonr">Next &gt;</' + 'div><div id="sw-done">Done</' + 'div></' + 'div><div id="sw-slots-wrapper"><div id="sw-slots"></' + 'div></' + 'div><div id="sw-frame"></' + 'div>';

		document.body.appendChild(div);

document.getElementById('sw-header').style.height=Math.floor(43*this.pixelRatio)+'px';
document.getElementById('sw-cancel').style.top=Math.floor(11*this.pixelRatio)+'px';
document.getElementById('sw-cancel').style.height=Math.floor(20*this.pixelRatio)+'px';
document.getElementById('sw-cancel').style.lineHeight=Math.floor(20*this.pixelRatio)+'px';
document.getElementById('sw-cancel').style.fontSize=Math.floor(13*this.pixelRatio)+'px';
document.getElementById('sw-done').style.top=Math.floor(11*this.pixelRatio)+'px';
document.getElementById('sw-done').style.height=Math.floor(20*this.pixelRatio)+'px';
document.getElementById('sw-done').style.lineHeight=Math.floor(20*this.pixelRatio)+'px';
document.getElementById('sw-done').style.fontSize=Math.floor(13*this.pixelRatio)+'px';
document.getElementById('sw-buttonl').style.top=Math.floor(11*this.pixelRatio)+'px';
document.getElementById('sw-buttonl').style.height=Math.floor(20*this.pixelRatio)+'px';
document.getElementById('sw-buttonl').style.lineHeight=Math.floor(20*this.pixelRatio)+'px';
document.getElementById('sw-buttonl').style.fontSize=Math.floor(13*this.pixelRatio)+'px';
document.getElementById('sw-buttonr').style.top=Math.floor(11*this.pixelRatio)+'px';
document.getElementById('sw-buttonr').style.height=Math.floor(20*this.pixelRatio)+'px';
document.getElementById('sw-buttonr').style.lineHeight=Math.floor(20*this.pixelRatio)+'px';
document.getElementById('sw-buttonr').style.fontSize=Math.floor(13*this.pixelRatio)+'px';
document.getElementById('sw-slots-wrapper').style.height=Math.floor(215*this.pixelRatio)+'px';
document.getElementById('sw-frame').style.height=Math.floor(183*this.pixelRatio)+'px';
//document.getElementById('sw-frame').style.borderWidth=Math.floor(16*this.pixelRatio)+'px';
//document.getElementById('sw-frame').style.webkitBorderImage='url(img/sw-alpha.png) '+Math.floor(16*this.pixelRatio);
document.getElementById('sw-frame').style.padding=Math.floor(16*this.pixelRatio)+'px';

		
		this.swWrapper = div;													// The SW wrapper
		this.swSlotWrapper = document.getElementById('sw-slots-wrapper');		// Slots visible area
		this.swSlots = document.getElementById('sw-slots');						// Pseudo table element (inner wrapper)
		this.swFrame = document.getElementById('sw-frame');						// The scrolling controller

		// Create HTML slot elements
		for (l = 0; l < this.slotData.length; l += 1) {
			// Create the slot
			ul = document.createElement('ul');
ul.style.paddingBottom=Math.floor(85*this.pixelRatio)+'px';			
			out = '';
			for (i in this.slotData[l].values) {
				out += '<li style="height:'+Math.floor(44*this.pixelRatio)+'px;line-height:'+Math.floor(44*this.pixelRatio)+'px;font-size:'+Math.floor(24*this.pixelRatio)+'px;">' + this.slotData[l].values[i] + '<' + '/li>';
			}
			ul.innerHTML = out;

			div = document.createElement('div');		// Create slot container
			div.className = this.slotData[l].style;		// Add styles to the container
//div.style.height = (119*this.pixelRatio)+'px'; // Wegen dummen Android Browser
div.style.height = Math.floor(129*this.pixelRatio)+'px'; // Wegen dummen Android Browser
div.style.paddingTop = Math.floor(86*this.pixelRatio)+'px'; 

			div.appendChild(ul);
			// Append the slot to the wrapper
			this.swSlots.appendChild(div);
			
			ul.slotPosition = l;			// Save the slot position inside the wrapper
			ul.slotYPosition = 0;
			ul.slotWidth = 0;
			ul.slotMaxScroll = this.swSlotWrapper.clientHeight - ul.clientHeight - (86*this.pixelRatio);
			
			ul.style.webkitTransitionTimingFunction = 'cubic-bezier(0, 0, 0.2, 1)';		// Add default transition
			
			this.slotEl.push(ul);			// Save the slot for later use
			
			// Place the slot to its default position (if other than 0)
			if (this.slotData[l].defaultValue) {
				this.scrollToValue(l, this.slotData[l].defaultValue);	
			}
		}
		
		this.calculateSlotsWidth();
		
		// Global events
		document.addEventListener('touchstart', this, false);			// Prevent page scrolling
		document.addEventListener('touchmove', this, false);			// Prevent page scrolling
		window.addEventListener('orientationchange', this, true);		// Optimize SW on orientation change
		window.addEventListener('scroll', this, true);				// Reposition SW on page scroll

		// Cancel/Done buttons events
		document.getElementById('sw-cancel').addEventListener('touchstart', this, false);
		document.getElementById('sw-done').addEventListener('touchstart', this, false);
		document.getElementById('sw-buttonl').addEventListener('touchstart', this, false);
		document.getElementById('sw-buttonr').addEventListener('touchstart', this, false);

		// Add scrolling to the slots
		this.swFrame.addEventListener('touchstart', this, false);
		
	},

	open: function () {
		this.create();

		this.swWrapper.style.webkitTransitionTimingFunction = 'ease-out, ease-out';
		this.swWrapper.style.webkitTransitionDuration = '400ms, 400ms';
		this.swWrapper.style.webkitTransform = 'translate3d(0, -'+(259*this.pixelRatio)+'px, 0)';
		this.swWrapper.style.opacity = 1;		
	},
	fastopen: function () {
		this.create();

		this.swWrapper.style.webkitTransitionTimingFunction = 'ease-out, ease-out';
		this.swWrapper.style.webkitTransitionDuration = '100ms, 100ms';
		this.swWrapper.style.webkitTransform = 'translate3d(0, -'+(259*this.pixelRatio)+'px, 0)';
		this.swWrapper.style.opacity = 1;
	},
	
	
	/**
	 *
	 * Unload
	 *
	 */

	destroy: function () {
		this.swWrapper.removeEventListener('webkitTransitionEnd', this, false);

		this.swFrame.removeEventListener('touchstart', this, false);

		document.getElementById('sw-cancel').removeEventListener('touchstart', this, false);
		document.getElementById('sw-done').removeEventListener('touchstart', this, false);
		document.getElementById('sw-buttonl').removeEventListener('touchstart', this, false);
		document.getElementById('sw-buttonr').removeEventListener('touchstart', this, false);

		document.removeEventListener('touchstart', this, false);
		document.removeEventListener('touchmove', this, false);
		window.removeEventListener('orientationchange', this, true);
		window.removeEventListener('scroll', this, true);
		
		this.slotData = [];
		this.cancelAction = function () {
			return false;
		};
		
		this.cancelDone = function () {
			return true;
		};

		this.cancelButtonl = function () {
			return true;
		};
		this.cancelButtonr = function () {
			return true;
		};
		
		this.reset();
		
		document.body.removeChild(document.getElementById('sw-wrapper'));

		// Schutzschicht
		if(document.getElementById('sw-shield'))document.body.removeChild(document.getElementById('sw-shield'));
		
		if(this.callBack){
			if(typeof this.callBack == 'function'){
				this.callBack();
			}else{
				eval(this.callBack)
			}
		}
	},
	
	close: function () {
		this.callBack=null;
		this.swWrapper.style.webkitTransitionTimingFunction = 'ease-in, ease-in';
		this.swWrapper.style.webkitTransitionDuration = '400ms, 400ms';
		this.swWrapper.style.webkitTransform = 'translate3d(0, 0, 0)';
		this.swWrapper.style.opacity = 0;
		
		this.swWrapper.addEventListener('webkitTransitionEnd', this, false);
	},
	fastclose: function (callBack) {
		this.callBack=callBack;
		this.swWrapper.style.webkitTransitionTimingFunction = 'ease-in, ease-in';
		this.swWrapper.style.webkitTransitionDuration = '100ms, 100ms';
		this.swWrapper.style.webkitTransform = 'translate3d(0, 0, 0)';
		this.swWrapper.style.opacity = 0;
		this.swWrapper.addEventListener('webkitTransitionEnd', this, false);
	},


	/**
	 *
	 * Generic methods
	 *
	 */

	addSlot: function (values, style, defaultValue) {
		if (!style) {
			style = '';
		}
		
		style = style.split(' ');

		for (var i = 0; i < style.length; i += 1) {
			style[i] = 'sw-' + style[i];
		}
		
		style = style.join(' ');

		var obj = { 'values': values, 'style': style, 'defaultValue': defaultValue };
		this.slotData.push(obj);
	},

	getSelectedValues: function () {
		var index, count,
		    i, l,
			keys = [], values = [];

		for (i in this.slotEl) {
			// Remove any residual animation
			this.slotEl[i].removeEventListener('webkitTransitionEnd', this, false);
			this.slotEl[i].style.webkitTransitionDuration = '0';

			if (this.slotEl[i].slotYPosition > 0) {
				this.setPosition(i, 0);
			} else if (this.slotEl[i].slotYPosition < this.slotEl[i].slotMaxScroll) {
				this.setPosition(i, this.slotEl[i].slotMaxScroll);
			}

			index = -Math.round(this.slotEl[i].slotYPosition / this.cellHeight);

			count = 0;
			for (l in this.slotData[i].values) {
				if (count == index) {
					keys.push(l);
					values.push(this.slotData[i].values[l]);
					break;
				}
				
				count += 1;
			}
		}

		return { 'keys': keys, 'values': values };
	},


	/**
	 *
	 * Rolling slots
	 *
	 */

	setPosition: function (slot, pos) {
		this.slotEl[slot].slotYPosition = pos;
		this.slotEl[slot].style.webkitTransform = 'translate3d(0, ' + pos + 'px, 0)';
// hk 11.07.2012
//		this.slotEl[slot].addEventListener('webkitTransitionEnd', this, false);
		// hk 11.07.2012 onSpin
		if(this.activeSlot&&this.onSpin){
			if(typeof this.onSpin == 'function'){
				this.onSpin(slot,pos);
			}else{
				eval(this.onSpin)
			}
		}

	},
	
	scrollStart: function (e) {
		// Find the clicked slot
		//var xPos = e.targetTouches[0].clientX - this.swSlots.offsetLeft;	// Clicked position minus left offset (should be 11px)
		var xPos = e.targetTouches[0].clientX - findPosX(this.swSlots);	// Clicked position minus left offset (should be 11px)
		
		// Find tapped slot
		var slot = 0;
		for (var i = 0; i < this.slotEl.length; i += 1) {
			slot += this.slotEl[i].slotWidth;
			
			if (xPos < slot) {
				this.activeSlot = i;
				break;
			}
		}
// hk 12.07.2012: Sehr spezieller Hack um bereits bei Berührung die Haken zu setzen		
AutoHakenOnSpin(this.activeSlot,0)
		// If slot is readonly do nothing
		if (this.slotData[this.activeSlot].style.match('readonly')) {
			this.swFrame.removeEventListener('touchmove', this, false);
			this.swFrame.removeEventListener('touchend', this, false);
			return false;
		}

		this.slotEl[this.activeSlot].removeEventListener('webkitTransitionEnd', this, false);	// Remove transition event (if any)
		this.slotEl[this.activeSlot].style.webkitTransitionDuration = '0';		// Remove any residual transition
		
		// Stop and hold slot position
		var theTransform = window.getComputedStyle(this.slotEl[this.activeSlot]).webkitTransform;
		theTransform = new WebKitCSSMatrix(theTransform).m42;
		if (theTransform != this.slotEl[this.activeSlot].slotYPosition) {
			this.setPosition(this.activeSlot, theTransform);
		}
		
		this.startY = e.targetTouches[0].clientY;
		this.scrollStartY = this.slotEl[this.activeSlot].slotYPosition;
		this.scrollStartTime = e.timeStamp;

		this.swFrame.addEventListener('touchmove', this, false);
		this.swFrame.addEventListener('touchend', this, false);
		
		return true;
	},

	scrollMove: function (e) {
		var topDelta = e.targetTouches[0].clientY - this.startY;

		if (this.slotEl[this.activeSlot].slotYPosition > 0 || this.slotEl[this.activeSlot].slotYPosition < this.slotEl[this.activeSlot].slotMaxScroll) {
			topDelta /= 2;
		}
		
		this.setPosition(this.activeSlot, this.slotEl[this.activeSlot].slotYPosition + topDelta);
		this.startY = e.targetTouches[0].clientY;

		// Prevent slingshot effect
		if (this.device=='i'&&e.timeStamp - this.scrollStartTime > 80) {
			this.scrollStartY = this.slotEl[this.activeSlot].slotYPosition;
			this.scrollStartTime = e.timeStamp;
		}
		
	},
	
	scrollEnd: function (e) {
		this.swFrame.removeEventListener('touchmove', this, false);
		this.swFrame.removeEventListener('touchend', this, false);

		// If we are outside of the boundaries, let's go back to the sheepfold
		if (this.slotEl[this.activeSlot].slotYPosition > 0 || this.slotEl[this.activeSlot].slotYPosition < this.slotEl[this.activeSlot].slotMaxScroll) {
			this.scrollTo(this.activeSlot, this.slotEl[this.activeSlot].slotYPosition > 0 ? 0 : this.slotEl[this.activeSlot].slotMaxScroll);
			return false;
		}

		// Lame formula to calculate a fake deceleration
		var scrollDistance = this.slotEl[this.activeSlot].slotYPosition - this.scrollStartY;

		// The drag session was too short
		if (scrollDistance < this.cellHeight / 1.5 && scrollDistance > -this.cellHeight / 1.5) {
			if (this.slotEl[this.activeSlot].slotYPosition % this.cellHeight) {
				this.scrollTo(this.activeSlot, Math.round(this.slotEl[this.activeSlot].slotYPosition / this.cellHeight) * this.cellHeight, '100ms');
			}

			return false;
		}

		var scrollDuration = e.timeStamp - this.scrollStartTime;

		var newDuration = (2 * scrollDistance / scrollDuration) / this.friction;
		var newScrollDistance = (this.friction / 2) * (newDuration * newDuration);
		
		if (newDuration < 0) {
			newDuration = -newDuration;
			newScrollDistance = -newScrollDistance;
		}

		var newPosition = this.slotEl[this.activeSlot].slotYPosition + newScrollDistance;

		if (newPosition > 0) {
			// Prevent the slot to be dragged outside the visible area (top margin)
			newPosition /= 2;
			newDuration /= 3;

			if (newPosition > this.swSlotWrapper.clientHeight / 4) {
				newPosition = this.swSlotWrapper.clientHeight / 4;
			}
		} else if (newPosition < this.slotEl[this.activeSlot].slotMaxScroll) {
			// Prevent the slot to be dragged outside the visible area (bottom margin)
			newPosition = (newPosition - this.slotEl[this.activeSlot].slotMaxScroll) / 2 + this.slotEl[this.activeSlot].slotMaxScroll;
			newDuration /= 3;
			
			if (newPosition < this.slotEl[this.activeSlot].slotMaxScroll - this.swSlotWrapper.clientHeight / 4) {
				newPosition = this.slotEl[this.activeSlot].slotMaxScroll - this.swSlotWrapper.clientHeight / 4;
			}
		} else {
			newPosition = Math.round(newPosition / this.cellHeight) * this.cellHeight;
		}

		this.scrollTo(this.activeSlot, Math.round(newPosition), Math.round(newDuration) + 'ms');
 
		return true;
	},

	scrollTo: function (slotNum, dest, runtime) {
		this.slotEl[slotNum].style.webkitTransitionDuration = runtime ? runtime : '100ms';
		this.setPosition(slotNum, dest ? dest : 0);

		// If we are outside of the boundaries go back to the sheepfold
		if (this.slotEl[slotNum].slotYPosition > 0 || this.slotEl[slotNum].slotYPosition < this.slotEl[slotNum].slotMaxScroll) {
			this.slotEl[slotNum].addEventListener('webkitTransitionEnd', this, false);
		}
	},
	
	scrollToValue: function (slot, value) {
		var yPos, count, i;

		this.slotEl[slot].removeEventListener('webkitTransitionEnd', this, false);
		this.slotEl[slot].style.webkitTransitionDuration = '0';
		
		count = 0;
		for (i in this.slotData[slot].values) {
			if (i == value) {
				yPos = count * this.cellHeight;
				this.setPosition(slot, yPos);
				break;
			}
			
			count -= 1;
		}
	},
	
	backWithinBoundaries: function (e) {
		e.target.removeEventListener('webkitTransitionEnd', this, false);

		this.scrollTo(e.target.slotPosition, e.target.slotYPosition > 0 ? 0 : e.target.slotMaxScroll, '150ms');
		return false;
	},


	/**
	 *
	 * Buttons
	 *
	 */

	tapDown: function (e) {
		e.currentTarget.addEventListener('touchmove', this, false);
		e.currentTarget.addEventListener('touchend', this, false);
		e.currentTarget.className = 'sw-pressed';
	},

	tapCancel: function (e) {
		e.currentTarget.removeEventListener('touchmove', this, false);
		e.currentTarget.removeEventListener('touchend', this, false);
		e.currentTarget.className = '';
	},
	
	tapUp: function (e) {
		this.tapCancel(e);
		if (e.currentTarget.id == 'sw-cancel') {
			this.cancelAction();
			this.close();
		} else {
			if (e.currentTarget.id == 'sw-done') {
				this.doneAction();
				this.close();
			} else {
				if (e.currentTarget.id == 'sw-buttonl') {
					this.buttonlAction();
				} else {
					this.buttonrAction();
				}
			}
		}
		
		//this.close();
	},

	setDevice: function (c) {
		this.device = c;
	},

	setButtonTexts: function (c1,c2,c3,c4) {
		if(c1!=null){
			if(c1!=''){
				document.getElementById('sw-cancel').innerHTML=c1;
				document.getElementById('sw-cancel').style.display='block';
			}else{
				document.getElementById('sw-cancel').style.display='none';
			}
		}
		if(c2!=null){
			if(c2!=''){
				document.getElementById('sw-done').innerHTML=c2;
				document.getElementById('sw-done').style.display='block';
			}else{
				document.getElementById('sw-done').style.display='none';
			}
		}
		if(c3!=null){
			if(c3!=''){
				document.getElementById('sw-buttonl').innerHTML=c3;
				document.getElementById('sw-buttonl').style.display='block';
			}else{
				document.getElementById('sw-buttonl').style.display='none';
			}
		}
		if(c4!=null){
			if(c4!=''){
				document.getElementById('sw-buttonr').innerHTML=c4;
				document.getElementById('sw-buttonr').style.display='block';
			}else{
				document.getElementById('sw-buttonr').style.display='none';
			}
		}

	},

	setCancelAction: function (action) {
		this.cancelAction = action;
	},

	setDoneAction: function (action) {
		this.doneAction = action;
	},

	setButtonlAction: function (action) {
		this.buttonlAction = action;
	},
	
	setButtonrAction: function (action) {
		this.buttonrAction = action;
	},
	
	cancelAction: function () {
		return false;
	},

	cancelDone: function () {
		return true;
	},

	cancelButtonl: function () {
		return true;
	},
	
	cancelButtonr: function () {
		return true;
	}

	
};